# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function

from ..errors import CheddarError
from .currency import Currency


class Payment(object):
    def __str__(self):
        return str(self.__dict__)

    def __init__(self, data):
        try:
            self.status = data["status"]["status"]

            self.uuid = data["uuid"]
            self.service = data["service"]["handle"]
            self.amount = data["amount"]
            self.refunded_amount = data["refunded_amount"]
            self.service_fee_amount = data["service_fee_amount"]
            self.variable_symbol = data["variable_symbol"]
            self.constant_symbol = data["constant_symbol"]
            self.currency = Currency(
                data["currency"]["name"],
                data["currency"]["alpha_code"],
                data["currency"]["numeric_code"],
            )

            self.periodicity = (
                data["periodicity"] if "periodicity" in data else 0
            )
            self.periodicity_no = (
                data["periodicity_no"] if "periodicity_no" in data else None
            )
            self.charge_on = data["charge_on"] if "charge_on" in data else None

            self.card_no = data["card_no"] if "card_no" in data else None
            self.card_expire_on = (
                data["card_expire_on"] if "card_expire_on" in data else None
            )

            self.transaction_identifier = (
                data["transaction_identifier"]
                if "transaction_identifier" in data
                else None
            )

            self._redirect_url = (
                data["redirect_url"] if "redirect_url" in data else None
            )
        except (KeyError, TypeError) as e:
            raise CheddarError(
                "Payment cannot be instantiated due to incorrect key: %s" % e
            )

    @property
    def redirect_url(self):
        if self.status != "none":
            raise CheddarError(
                "Further processing is not possible due to unprocessable status %s (The only valid payment status for further processing is 'none')"
                % self.status
            )

        if not self._redirect_url or not self.uuid:
            raise CheddarError(
                "Further processing is not possible due to incomplete payment object."
            )

        return self._redirect_url



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function


class CheddarError(Exception):
    def __init__(self, message=None, http_body=None, http_status=None):
        super(CheddarError, self).__init__(message)

        self.http_body = http_body
        self.http_status = http_status


class ConfigurationError(CheddarError):
    pass


class AuthenticationError(CheddarError):
    pass


class APIError(CheddarError):
    pass


class APIConnectionError(APIError):
    pass


class MessageIntegrityError(CheddarError):
    pass



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function

from .resources.transaction import Transaction


class Transactions(object):
    def __init__(self, client):
        self.requestor = client.requestor
        self.configuration = client.configuration

    def all(self):
        status_code, response = self.requestor.request("get", "/api/v1/transactions/")
        return list(map(Transaction, response.get('transactions', [])))

    def detail(self, uuid):
        status_code, response = self.requestor.request("get", "/api/v1/transactions/%s" % uuid)
        return Transaction(response)

    def update(self, uuid, metadata):
        raise NotImplemented("Transaction update is not implemented")



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function

from .errors import MessageIntegrityError


class Messages(object):
    def __init__(self, client):
        self.requestor = client.requestor

    def validate(self, uuid, signature):
        if signature == self.requestor.sign(uuid):
            return True

        raise MessageIntegrityError(
            "Signature %s for UUID %s is incorrect. Message is not to be trusted"
            % (signature, uuid)
        )



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function

from .configuration import Configuration
from .cheddar import Cheddar
from .environment import Environment
from .errors import (
    CheddarError,
    APIConnectionError,
    APIError,
    MessageIntegrityError,
)
from .resources.currency import Currency
from .resources.payment import Payment
from .resources.service import Service
from .utils import JSONEncoder



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function


class Service(object):
    SPOROPAY = "sporopay"
    ECARD = "ecard"
    EPLATBY = "eplatby"
    TATRAPAY = "tatrapay"
    CARDPAY = "cardpay"
    PAYPAL = "paypal"
    GPWEBPAY = "gpwebpay"
    ITERMINAL = "iterminal"



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function

import cheddar
import json

from datetime import datetime, date
from decimal import Decimal


class TestJSONEncoder(object):
    def test_encoding_decimal(self):
        data = {"amount": Decimal("2.27")}
        output = json.dumps(data, cls=cheddar.utils.JSONEncoder)
        assert output == '{"amount": "2.27"}'

    def test_encoding_date(self):
        data = {"charge_on": date(2019, 5, 9)}
        output = json.dumps(data, cls=cheddar.utils.JSONEncoder)
        assert output == '{"charge_on": "2019-05-09"}'

    def test_encoding_datetime_as_date(self):
        data = {"charge_on": datetime(2019, 5, 9, 19, 32, 46)}
        output = json.dumps(data, cls=cheddar.utils.JSONEncoder)
        assert output == '{"charge_on": "2019-05-09"}'



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function

from .currency import Currency


class Transaction(object):
    def __init__(self, data):
        self.uuid = data['uuid']
        self.type = data['type']
        self.payment_uuid = data['payment']
        self.bank_identifier = data['bank_identifier']

        self.variable_symbol = data['variable_symbol'] if 'variable_symbol' in data else None
        self.constant_symbol = data['constant_symbol'] if 'constant_symbol' in data else None
        self.specific_symbol = data['specific_symbol'] if 'specific_symbol' in data else None

        self.amount = data['amount']
        self.currency = Currency(
            data["currency"]["name"],
            data["currency"]["alpha_code"],
            data["currency"]["numeric_code"],
        )

        self.iban = data['iban'] if 'iban' in data else None
        self.notes = data['notes'] if 'notes' in data else None
        self.description = data['description'] if 'description' in data else None

        self.booked_at = data['booked_at'] if 'booked_at' in data else None
        self.created_at = data['created_at'] if 'created_at' in data else None

    def __str__(self):
        return str(self.__dict__)



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function

from .resources.payment import Payment
from datetime import date


class Payments(object):
    def __init__(self, client):
        self.requestor = client.requestor
        self.configuration = client.configuration

    def details(self, uuid):
        status_code, response = self.requestor.request(
            "get", "/api/v1/payments/%s/" % (uuid)
        )

        return Payment(response)

    def create(self, service, metadata):
        data = {"service": service, "metadata": metadata}

        if "payer_ip_address" not in data["metadata"]:
            data["metadata"]["payer_ip_address"] = self.requestor.client_ip(
                self.configuration.os_environ
            )

        status_code, response = self.requestor.request(
            "post", "/api/v1/payments/", data
        )

        return Payment(response)

    def update(self, uuid, metadata):
        if "charge_on" in metadata and isinstance(metadata["charge_on"], date):
            metadata["charge_on"] = metadata["charge_on"].isoformat()

        data = {"metadata": metadata}

        status_code, response = self.requestor.request(
            "put", "/api/v1/payments/%s/" % uuid, data
        )

        return Payment(response)

    def refund(self, uuid, refund):
        data = {"refund": refund}

        status_code, response = self.requestor.request(
            "post", "/api/v1/payments/%s/refund" % uuid, data
        )

        return Payment(response)



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function


class Currency(object):
    CZK = "CZK"
    DKK = "DKK"
    EUR = "EUR"
    GBP = "GBP"
    HUF = "HUF"
    CHF = "CHF"
    PLN = "PLN"
    USD = "USD"

    def __init__(self, name, alpha_code, numeric_code):
        self.name = name
        self.alpha_codde = alpha_code
        self.numeric_code = numeric_code



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function

import os

from .environment import Environment
from .errors import ConfigurationError


class Configuration(object):
    def __init__(self, environment, key, secret, *args, **kwargs):
        self.environment = Environment.parse(environment)

        if key == "":
            raise ConfigurationError("Missing public key")
        else:
            self.key = key

        if secret == "":
            raise ConfigurationError("Missing secret")
        else:
            self.secret = secret

        self.timeout = kwargs.get("timeout", 30)
        self.os_environ = kwargs.get("os_environ", os.environ)



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function
from .errors import ConfigurationError


class Environment(object):
    def __init__(self, name, api_base_url):
        self.name = name
        self.api_base_url = api_base_url

    @staticmethod
    def parse(environment=None):
        if isinstance(environment, Environment) or environment is None:
            return environment
        else:
            raise ConfigurationError("Unable to process supplied environment")

    def __str__(self):
        return self.name


Environment.Sandbox = Environment(
    "sandbox", "https://sandbox.cheddarpayments.com"
)
Environment.Production = Environment(
    "production", "https://www.cheddarpayments.com"
)
